{***********************************************************************}
{ TMS Security framework                                                }
{ for Delphi & C++Builder                                               }
{                                                                       }
{ written by                                                            }
{   TMS Software                                                        }
{   Copyright  2003 - 2012                                             }
{   Email : info@tmssoftware.com                                        }
{   Web : http://www.tmssoftware.com                                    }
{***********************************************************************}

unit uIWFormPolicyDesigner;

{$I TMSDEFS.INC}

interface

uses
  Classes, SysUtils, IWAppForm, IWApplication, IWColor, IWTypes,
  IWCompButton, Controls, IWVCLBaseControl, IWBaseControl, IWBaseHTMLControl,
  IWControl, IWCompLabel, uFormPolicy, ImgList, IWCompEdit, Windows, DB,
  Forms, IWVCLBaseContainer, IWContainer, IWHTMLContainer, IWHTML40Container,
  IWRegion, IWServer
  {$IFDEF TMSIW121}
  , IWCompTreeview, IWCompExtCtrls
  {$ENDIF}
  {$IFNDEF TMSIW121}
  , IWTreeview, IWExtCtrls
  {$ENDIF}
  ;

type
  TIWFormPolicyDesigner = class(TIWAppForm)
    IWRegion1: TIWRegion;
    tvPolicies: TIWTreeView;
    tvComponents: TIWTreeView;
    btAddPolicy: TIWButton;
    btDelPolicy: TIWButton;
    rgPolicyType: TIWRadioGroup;
    btAddComp: TIWButton;
    btDelComp: TIWButton;
    btOK: TIWButton;
    btCancel: TIWButton;
    lbPolicies: TIWLabel;
    lbComponents: TIWLabel;
    edAddPolicy: TIWEdit;
    IWRegion2: TIWRegion;
    lbFormPolicy: TIWLabel;
    lbWarning: TIWLabel;
    IWImageFile1: TIWImageFile;
    procedure btOKClick(Sender: TObject);
    procedure btCancelClick(Sender: TObject);
    procedure tvPoliciesTreeItemClick(Sender: TObject;
      ATreeViewItem: TIWTreeViewItem);
    procedure tvComponentsTreeItemClick(Sender: TObject;
      ATreeViewItem: TIWTreeViewItem);
    procedure IWAppFormCreate(Sender: TObject);
    procedure btDelPolicyClick(Sender: TObject);
    procedure btAddPolicyClick(Sender: TObject);
    procedure btAddCompClick(Sender: TObject);
    procedure btDelCompClick(Sender: TObject);
    procedure rgPolicyTypeClick(Sender: TObject);
    procedure edAddPolicyAsyncKeyUp(Sender: TObject; EventParams: TStringList);
    procedure IWAppFormRender(Sender: TObject);
  private
    FUILFormPolicy: TUilFormPolicy;
    FFormName: string;
    PoliciesIndex, ComponentsIndex: integer;
    procedure SelectTVItem(ATreeViewItem: TIWTreeViewItem; ATreeView: TIWTreeView; SelectedIndex: integer);
    procedure SavePolicies;
  public
    FUilPolicy: TuilPolicy;
    ParentFormName: string;
    property FormPolicy: TUilFormPolicy read FUilFormPolicy write FUilFormPolicy;
    property FormName: string read FFormName write FFormName;
  end;

implementation

{$R *.dfm}

uses
  TypInfo;


procedure TIWFormPolicyDesigner.btAddCompClick(Sender: TObject);
var
  ATreeViewItem: TIWTreeViewItem;
  myNames: TStringList;
begin
  if (PoliciesIndex >= 0) and (ComponentsIndex >= 0) then
  begin
    ATreeViewItem := tvPolicies.Items.Add(tvPolicies.Items[PoliciesIndex]);
    ATreeViewItem.Caption := tvComponents.Items[ComponentsIndex].Caption;

    myNames := TStringList.Create;
    myNames.Add(TStringList(tvComponents.Items[ComponentsIndex].UserData)[0]);
    ATreeViewItem.UserData := myNames;
//  generates invalid pointer error
//    ATreeViewItem.UserData := tvComponents.Items[ComponentsIndex].UserData;
    ATreeViewItem.Tag := -1;

    ATreeViewItem.ItemImages.DocumentImage.URL := '/files/component.gif';

    SelectTVItem(ATreeViewItem, tvPolicies, PoliciesIndex);
//    PoliciesIndex := ATreeViewItem.Index;
  end;
end;

procedure TIWFormPolicyDesigner.btAddPolicyClick(Sender: TObject);
var
  ATreeViewItem: TIWTreeViewItem;
  I: Integer;
  found: boolean;
  myNames: TStringList;
begin
  if edAddPolicy.Text <> EmptyStr then
  begin
    found := false;
    for I := 0 to tvPolicies.Items.Count - 1 do
    begin
      if (tvPolicies.Items[i].ParentItem = nil)
        and (LowerCase(TStringList(tvPolicies.Items[i].UserData)[0]) = LowerCase(edAddPolicy.Text)) then
        begin
          found := true;
          tvPolicies.ScrollToName := tvPolicies.Items[i].Name;
        end;
    end;

    if found then
      lbWarning.Text := 'Policyname already exists.'
    else
    begin
      ATreeViewItem := tvPolicies.Items.Add(nil);
      ATreeViewItem.Caption := edAddPolicy.Text;
      myNames := TStringList.Create;
      myNames.Add(ATreeViewItem.Caption);
      ATreeViewItem.UserData := myNames;
//      ATreeViewItem.UserData := ATreeViewItem.UserData;
      ATreeViewItem.Tag := -1;
      SelectTVItem(ATreeViewItem, tvPolicies, PoliciesIndex);
      PoliciesIndex := ATreeViewItem.Index;
      tvPolicies.Selected := ATreeViewItem;

      tvPolicies.ScrollToName := ATreeViewItem.Name;

      lbWarning.Text := EmptyStr;
    end;
    
  end;
end;

procedure TIWFormPolicyDesigner.btCancelClick(Sender: TObject);
begin
  TIWAppForm(WebApplication.ActiveForm).Release;
end;

procedure TIWFormPolicyDesigner.btDelCompClick(Sender: TObject);
begin
  if PoliciesIndex >= 0 then
  begin
    tvPolicies.Items.Delete(PoliciesIndex);
    PoliciesIndex := -1;
  end;
end;

procedure TIWFormPolicyDesigner.btDelPolicyClick(Sender: TObject);
begin
  if PoliciesIndex >= 0 then
  begin

    if FUilPolicy.FormPolicy.SecurityManager.PermissionBindary.Locate(TStringList(tvPolicies.Selected.UserData)[0])
      and Assigned(FormPolicy) then
      FormPolicy.RemoveFromDataBase(FormName, TStringList(tvPolicies.Selected.UserData)[0]);

    tvPolicies.Items.Delete(PoliciesIndex);
    PoliciesIndex := -1;
  end;
end;

procedure TIWFormPolicyDesigner.btOKClick(Sender: TObject);
begin
  SavePolicies;
  TIWAppForm(WebApplication.ActiveForm).Release;
end;

procedure TIWFormPolicyDesigner.edAddPolicyAsyncKeyUp(Sender: TObject;
  EventParams: TStringList);
begin
  if edAddPolicy.Text <> EmptyStr then
    btAddPolicy.Enabled := true
  else
    btAddPolicy.Enabled := false;
end;

procedure TIWFormPolicyDesigner.IWAppFormCreate(Sender: TObject);
var
  PropInfo: PPropInfo;
begin
  PropInfo := TypInfo.GetPropInfo(rgPolicyType.ClassInfo,'RawText');
  if (PropInfo<>nil) then
   SetOrdProp(rgPolicyType,PropInfo,NativeInt(true));

  PoliciesIndex := -1;
  ComponentsIndex := -1;
  rgPolicyType.ItemIndex := -1;
  lbWarning.Text := EmptyStr;
  btAddPolicy.Enabled := false;
end;

procedure TIWFormPolicyDesigner.IWAppFormRender(Sender: TObject);
begin
  if PoliciesIndex < 0 then
  begin
    btDelPolicy.Enabled := false;
    rgPolicyType.Enabled := false;

    btDelComp.Enabled := false;
  end
  else
  begin
    tvPolicies.Selected := tvPolicies.Items[PoliciesIndex];
    tvPolicies.ScrollToName := tvPolicies.Selected.Name;

    if tvPolicies.Selected.ParentItem = nil then
    begin
      btDelPolicy.Enabled := true;
      rgPolicyType.Editable := true;

      btDelComp.Enabled := false;
    end
    else
    begin
      btDelPolicy.Enabled := false;
      rgPolicyType.Editable := false;

      btDelComp.Enabled := true;
    end;
  end;

  if (PoliciesIndex < 0) or (ComponentsIndex < 0)
    or (tvPolicies.Selected.ParentItem <> nil)
    or (tvComponents.Selected.ParentItem = nil) then
    btAddComp.Enabled := false
  else
    btAddComp.Enabled := true;
  
end;

procedure TIWFormPolicyDesigner.rgPolicyTypeClick(Sender: TObject);
begin
  if PoliciesIndex >= 0 then
  begin
    with tvPolicies.Items[PoliciesIndex] do
    begin
      Tag := rgPolicyType.ItemIndex;
      case rgPolicyType.ItemIndex of
        0: ItemImages.OpenFolderImage.URL := '/files/disable.gif';
        1: ItemImages.OpenFolderImage.URL := '/files/hide.gif';
        2: ItemImages.OpenFolderImage.URL := '/files/custom.gif';
        3: ItemImages.OpenFolderImage.URL := '/files/readonly.gif';
      end;
      ItemImages.ClosedFolderImage.URL := ItemImages.OpenFolderImage.URL;
      ItemImages.DocumentImage.URL := ItemImages.OpenFolderImage.URL;
    end;
  end;
end;

procedure TIWFormPolicyDesigner.SavePolicies;
var
  Item: TuilPolicyItem;
  Node, SubNode: TIWTreeViewItem;
  OldActive: boolean;
  ItemList: string;
  I: Integer;
  J: Integer;
begin
  OldActive := FUilPolicy.FormPolicy.SecurityManager.PermissionBindary.DataSource.DataSet.Active;
  if not OldActive then
    FUilPolicy.FormPolicy.SecurityManager.PermissionBindary.DataSource.DataSet.Open;
  try
    FUilPolicy.Clear;

    for I := 0 to tvPolicies.Items.Count - 1 do
    begin
      Node := tvPolicies.Items[i];

      if Node.ParentItem = nil then
      begin
        Item := FUilPolicy.Add as TuilPolicyItem;
        Item.PolicyName := TStringList(Node.UserData)[0];
  //      Item.PolicyAction := TPolicyAction(Node.ImageIndex);
        Item.PolicyAction := TPolicyAction(Node.Tag);

        ItemList := '';
        //SubNode := Node.GetFirstChild;
        //SubNode := Node.SubItems[0];

        for J := I to tvPolicies.Items.Count - 1 do
        begin
          SubNode := tvPolicies.Items[j];

          if SubNode.ParentItem = Node then
          begin
          if ItemList <> '' then
            ItemList := ItemList + ',' + TStringList(SubNode.UserData)[0]
          else
            ItemList := TStringList(SubNode.UserData)[0];

          Item.Items.Add(TStringList(SubNode.UserData)[0]);
          end;

        end;  // for

  //      FUilPolicy.FormPolicy.SecurityManager.PermissionBindary.AddPermission(Item.PolicyName, ItemList, self.Owner.Name,Integer(Item.PolicyAction));
        FUilPolicy.FormPolicy.SecurityManager.PermissionBindary.AddPermission(Item.PolicyName, ItemList, ParentFormName, Integer(Item.PolicyAction));

      end;

    end; // for
  finally
    if not OldActive then
      FUilPolicy.FormPolicy.SecurityManager.PermissionBindary.DataSource.DataSet.Close;
  end;
end;

procedure TIWFormPolicyDesigner.SelectTVItem(ATreeViewItem: TIWTreeViewItem;
  ATreeView: TIWTreeView; SelectedIndex: integer);
begin
  if SelectedIndex >= 0 then
  begin
    with ATreeView.Items[SelectedIndex] do
    begin
      Font.Color := clWebBLUE;
      Caption := TStringList(UserData)[0];
    end;
  end;

  ATreeViewItem.Font.Color := clWebWhite;
  ATreeViewItem.Caption := '<span style="background-color:blue">'
    + ATreeViewItem.Caption + '</span>';
end;

procedure TIWFormPolicyDesigner.tvComponentsTreeItemClick(Sender: TObject;
  ATreeViewItem: TIWTreeViewItem);
begin
  SelectTVItem(ATreeViewItem, tvComponents, ComponentsIndex);
  ComponentsIndex := ATreeViewItem.Index;

  if ComponentsIndex >= 0 then
  begin
    tvComponents.Selected := tvComponents.Items[ComponentsIndex];
//    OutputDebugString(pchar('comp: ' + TStringList(tvComponents.Selected.UserData)[0] + ' * ' + tvComponents.Selected.Name));
    tvComponents.ScrollToName := tvComponents.Selected.Name;
  end;

end;

procedure TIWFormPolicyDesigner.tvPoliciesTreeItemClick(Sender: TObject;
  ATreeViewItem: TIWTreeViewItem);
begin
  SelectTVItem(ATreeViewItem, tvPolicies, PoliciesIndex);
  PoliciesIndex := ATreeViewItem.Index;

  if ATreeViewItem.ParentItem = nil then
  begin
    rgPolicyType.Enabled := true;
    rgPolicyType.ItemIndex := ATreeViewItem.Tag;
  end
  else
  begin
    rgPolicyType.Enabled := false;
    rgPolicyType.ItemIndex := -1;    
  end;
end;

initialization
  TIWFormPolicyDesigner.SetAsMainForm;
  
end.
