{***********************************************************************}
{ TMS Security System                                                   }
{ for Delphi & C++Builder                                               }
{                                                                       }
{ written by                                                            }
{   TMS Software                                                        }
{   Copyright  2003 - 2012                                             }
{   Email : info@tmssoftware.com                                        }
{   Web : http://www.tmssoftware.com                                    }
{***********************************************************************}

unit uIWFormPolicyDlg;

{$I TMSDEFS.INC}

interface

{$R TMSIWSS.res}

uses
  Windows, Classes, Dialogs, Controls, Forms, uFormPolicy, ComCtrls, ImgList,
  uFormPolicyDesigner, uSecControl, usecdlgform, TypInfo, SysUtils
  , uIWFormPolicyDesigner, IWAppForm, IWServer
  {$IFDEF TMSIW121}
  , IWCompTreeview, IWServerInternalFiles
  {$ENDIF}
  {$IFNDEF TMSIW121}
  , IWTreeview
  {$ENDIF}
  ;

type

  TControlItem = class(TCollectionItem)
  private
    FComponent: TComponent;
  protected
    function GetDisplayName: string; override;
  public
    constructor Create(Collection: TCollection); override;
    destructor Destroy; override;
    procedure Assign(Source: TPersistent); override;
  published
    property Component: TComponent read FComponent write FComponent;
  end;

  TControlCollection = class(TCollection)
  private
    FOwner: TComponent;
    function GetItem(Index: Integer): TControlItem;
    procedure SetItem(Index: Integer; const Value: TControlItem);
  public
    constructor Create(AOwner: TComponent);
    function Add: TControlItem;
    function Insert(index: Integer): TControlItem;
    property Items[Index: Integer]: TControlItem read GetItem write SetItem; default;
    function IsIgnored(Component: TComponent): Boolean;
    function GetOwner: TPersistent; override;    
  end;

  TuilFilterComponentEvent = procedure(Sender: TObject; AComponent: TComponent; var Allow: boolean) of object;

  TuilIWFormPolicyDlg = class(TComponent)
  private
    FUilFormPolicy: TUilFormPolicy;
    FIgnoreList: TControlCollection;
    FFormName: string;
    FUilFilterComponent: TuilFilterComponentEvent;
  protected
    procedure AddToList(AForm: TIWFormPolicyDesigner; CompName, ClassName: string);
    procedure AddComponents(EditForm: TIWAppForm; AForm: TIWFormPolicyDesigner);
    procedure AddPolicies(EditForm: TIWAppForm; AForm: TIWFormPolicyDesigner; APolicy: TuilPolicy);
    function GetClassImage(AClassName: string): integer;
  public
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    procedure Execute;
  published
    property FormPolicy: TUilFormPolicy read FUilFormPolicy write FUilFormPolicy;
    property FormName: string read FFormName write FFormName;
    property IgnoreList: TControlCollection read FIgnoreList write FIgnoreList;
    property OnFilterComponent: TuilFilterComponentEvent read FUilFilterComponent write FUilFilterComponent;
  end;

implementation

{ TuilFormPolicyDlg }

function TuilIWFormPolicyDlg.GetClassImage(AClassName: string): integer;
var
  ClassName: string;
begin
  Result := Unknown;
  ClassName := Uppercase(AClassName);
  if (ClassName = 'TLABEL') or (ClassName = 'TDBTEXT') then
    Result := LabelImage
  else if (ClassName = 'TBUTTON') or (ClassName = 'TBITBTN') then
    Result := ButtonImage
  else if (ClassName = 'TEDIT') or (ClassName = 'TDBEDIT') then
    Result := EditImage
  else if (ClassName = 'TCHECKBOX') or (ClassName = 'TDBCHECKBOX') then
    Result := CheckImage
  else if ClassName = 'TRADIOBUTTON' then
    Result := RadioImage
  else if ClassName = 'TMENUITEM' then
    Result := MenuImage
  else if (ClassName = 'TLISTBOX') or (ClassName = 'TDBLISTBOX') then
    Result := ListBoxImage
  else if (ClassName = 'TMEMO') or (ClassName = 'TDBMEMO') then
    Result := MemoImage;
end;

procedure TuilIWFormPolicyDlg.AddPolicies(EditForm: TIWAppForm; AForm: TIWFormPolicyDesigner; APolicy: TuilPolicy);
var
  k: integer;
  SubNode,
  Node: TIWTreeViewItem;
  j: integer;
  myNames: TStringList;
begin

  for j := 0 to APolicy.Count - 1 do // Iterate
  begin
    Node := AForm.tvPolicies.Items.Add(nil);
    Node.Caption := (APolicy.Items[j] as TuilPolicyItem).PolicyName;
    Node.Tag := Ord((APolicy.Items[j] as TuilPolicyItem).PolicyAction);

    myNames := TStringList.Create;
    myNames.Add(Node.Caption);
    Node.UserData := myNames;
//    myNames.Free;

    case Node.Tag of
    0:  Node.ItemImages.OpenFolderImage.URL := '/files/disable.gif';
    1:  Node.ItemImages.OpenFolderImage.URL := '/files/hide.gif';
    2:  Node.ItemImages.OpenFolderImage.URL := '/files/custom.gif';
    3:  Node.ItemImages.OpenFolderImage.URL := '/files/readonly.gif';
    end;
    Node.ItemImages.ClosedFolderImage.URL := Node.ItemImages.OpenFolderImage.URL;
    Node.ItemImages.DocumentImage.URL := Node.ItemImages.OpenFolderImage.URL;

    for k := 0 to (APolicy.Items[j] as TuilPolicyItem).Items.Count - 1 do // Iterate
    begin
        SubNode := AForm.tvPolicies.Items.Add(Node);
        SubNode.Caption := ((APolicy.Items[j] as TuilPolicyItem).Items[k]);

        myNames := TStringList.Create;
        myNames.Add(SubNode.Caption);
        SubNode.UserData := myNames;
//        myNames.Free;

        SubNode.ItemImages.DocumentImage.URL := '/files/component.gif';

        SubNode.Tag := -1;
    end;
  end;
end;

procedure TuilIWFormPolicyDlg.AddToList(AForm: TIWFormPolicyDesigner; CompName, ClassName: string);
var
  Found: boolean;
  Node, SubNode, newItem: TIWTreeViewItem;
  //ImageNumber,
  i: integer;
  myNames, myNames2, myNames3, myNames4, myNames5: TStringList;
begin

  if (CompName = '') or (ClassName = '') then Exit;

  Found := False;

  if (AForm.tvComponents.Items.Count = 0) then
  begin
    Node := AForm.tvComponents.Items.Add(nil);
    Node.Caption := ClassName;

    myNames := TStringList.Create;
    myNames.Add(Node.Caption);
    Node.UserData := myNames;
//    myNames.Free;

    Node.ItemImages.OpenFolderImage.URL := '/files/comptype.gif';
    Node.ItemImages.ClosedFolderImage.URL := Node.ItemImages.OpenFolderImage.URL;
    Node.ItemImages.DocumentImage.URL := Node.ItemImages.OpenFolderImage.URL;    

    SubNode := AForm.tvComponents.Items.Add(Node);
    SubNode.Caption := CompName;

    myNames2 := TStringList.Create;
    myNames2.Add(SubNode.Caption);
    SubNode.UserData := myNames2;
//    myNames.Free;

    SubNode.ItemImages.DocumentImage.URL := '/files/component.gif';

  end
  else
  begin
    for I := 0 to AForm.tvComponents.Items.Count - 1 do
    begin
      Node := AForm.tvComponents.Items[i];
      if TStringList(Node.UserData)[0] = ClassName then
      begin
        Found := True;
        newItem := AForm.tvComponents.Items.Add(Node);
        newItem.Caption := CompName;

        myNames3 := TStringList.Create;
        myNames3.Add(newItem.Caption);
        newItem.UserData := myNames3;

        newItem.ItemImages.DocumentImage.URL := '/files/component.gif';
//        myNames.Free;

      end;
    end;

    if Found = False then
    begin
      Node := AForm.tvComponents.Items.Add(nil);
      Node.Caption := ClassName;

      myNames4 := TStringList.Create;
      myNames4.Add(Node.Caption);
      Node.UserData := myNames4;
//      myNames.Free;

      Node.ItemImages.OpenFolderImage.URL := '/files/comptype.gif';
      Node.ItemImages.ClosedFolderImage.URL := Node.ItemImages.OpenFolderImage.URL;
      Node.ItemImages.DocumentImage.URL := Node.ItemImages.OpenFolderImage.URL;

      SubNode := AForm.tvComponents.Items.Add(Node);
      SubNode.Caption := CompName;

      myNames5 := TStringList.Create;
      myNames5.Add(SubNode.Caption);
      SubNode.UserData := myNames5;
//      myNames.Free;

      SubNode.ItemImages.DocumentImage.URL := '/files/component.gif';

    end;

  end;

  for I := 0 to AForm.tvComponents.Items.Count - 1 do
  begin
    AForm.tvComponents.Items[i].Name := 'li' + IntToStr(i);
  end;

end;


procedure TuilIWFormPolicyDlg.AddComponents(EditForm: TIWAppForm; AForm: TIWFormPolicyDesigner);
var
  j: integer;
  PropInfo: PPropInfo;
  Allow: boolean;
  
begin
  for j := 0 to EditForm.ComponentCount - 1 do // Iterate
  begin
    if not IgnoreList.IsIgnored(EditForm.Components[j]) then
    begin
      // find all components with Enabled, Visible or ReadOnly property
      PropInfo := TypInfo.GetPropInfo(EditForm.Components[j].ClassInfo, 'Enabled');
      if PropInfo <> nil then
      begin
        Allow := true;

        if Assigned(OnFilterComponent) then
          OnFilterComponent(Self,EditForm.Components[j], Allow);

        if Allow then
          AddToList(AForm, EditForm.Components[j].Name, EditForm.Components[j].ClassName)
      end
      else
      begin
        PropInfo := TypInfo.GetPropInfo(EditForm.Components[j].ClassInfo, 'Visible');
        if PropInfo <> nil then
        begin
          Allow := true;

          if Assigned(OnFilterComponent) then
            OnFilterComponent(Self,EditForm.Components[j], Allow);

          AddToList(AForm, EditForm.Components[j].Name, EditForm.Components[j].ClassName);
        end
        else
        begin
          PropInfo := TypInfo.GetPropInfo(EditForm.Components[j].ClassInfo, 'ReadOnly');
          if PropInfo <> nil then
          begin
            Allow := true;

            if Assigned(OnFilterComponent) then
              OnFilterComponent(Self,EditForm.Components[j], Allow);

            AddToList(AForm, EditForm.Components[j].Name, EditForm.Components[j].ClassName);
          end;
        end;
      end;
    end;
  end; // for

end;


procedure TuilIWFormPolicyDlg.Execute;
var
  AForm: TIWFormPolicyDesigner;
  Policies: TuilPolicy;
begin
  AForm := TIWFormPolicyDesigner.Create(nil);

  with AForm do
  begin
    try
      FormPolicy := self.FormPolicy;
      FormName := self.FormPolicy.Owner.Name;
      Policies := FuilFormPolicy.Policies;

      // Load up existing policies...
      AddComponents(self.Owner as TIWAppForm, AForm);
      FuilFormPolicy.LoadFromDatabase(FuilFormPolicy.Owner.Name);
      AddPolicies(self.Owner as TIWAppForm, AForm, Policies);
      AForm.FUilPolicy := Policies;
      AForm.ParentFormName := self.Owner.Name;

    finally
      //AForm.Free; { Free dialog.}
    end;
  end;

  AForm.Show;
    
end;


constructor TuilIWFormPolicyDlg.Create(AOwner: TComponent);
begin
  inherited;
  FIgnoreList := TControlCollection.Create(Self);

  {$IFDEF TMSIW121}
  gInternalFiles.add('IW_GFX_user', '/files/disable.gif');
  gInternalFiles.add('IW_GFX_disable', '/files/disable.gif');
  gInternalFiles.add('IW_GFX_hide', '/files/hide.gif');
  gInternalFiles.add('IW_GFX_custom', '/files/custom.gif');
  gInternalFiles.add('IW_GFX_readonly', '/files/readonly.gif');
  gInternalFiles.add('IW_GFX_component', '/files/component.gif');
  gInternalFiles.add('IW_GFX_comptype', '/files/comptype.gif');
  gInternalFiles.add('IW_GFX_policy', '/files/policy.gif');
  {$ELSE}
  TIWServer.AddInternalFile('IW_GFX_disable', '/files/disable.gif');
  TIWServer.AddInternalFile('IW_GFX_hide', '/files/hide.gif');
  TIWServer.AddInternalFile('IW_GFX_custom', '/files/custom.gif');
  TIWServer.AddInternalFile('IW_GFX_readonly', '/files/readonly.gif');
  TIWServer.AddInternalFile('IW_GFX_component', '/files/component.gif');
  TIWServer.AddInternalFile('IW_GFX_comptype', '/files/comptype.gif');
  TIWServer.AddInternalFile('IW_GFX_policy', '/files/policy.gif');
  {$ENDIF}
end;

destructor TuilIWFormPolicyDlg.Destroy;
begin
  FIgnoreList.Free;
  inherited;
end;

{ TControlItem }

procedure TControlItem.Assign(Source: TPersistent);
begin
  FComponent := (Source as TControlItem).Component;
end;

constructor TControlItem.Create(Collection: TCollection);
begin
  inherited Create(Collection);
end;

destructor TControlItem.Destroy;
begin
  inherited;
end;

function TControlItem.GetDisplayName: string;
begin
  if Assigned(Component) then
    Result := Component.Name
  else
    Result := 'Item '+ IntToStr(Index);  
end;

{ TControlCollection }

function TControlCollection.Add: TControlItem;
begin
  Result := TControlItem(inherited Add);
end;

constructor TControlCollection.Create(AOwner: TComponent);
begin
  inherited Create(TControlItem);
  FOwner := AOwner;
end;

function TControlCollection.GetItem(Index: Integer): TControlItem;
begin
  Result := TControlItem(inherited Items[Index]);
end;

function TControlCollection.GetOwner: TPersistent;
begin
  Result := FOwner;
end;

function TControlCollection.Insert(Index: Integer): TControlItem;
begin
  Result := TControlItem(inherited Insert(Index));
end;

function TControlCollection.IsIgnored(Component: TComponent): Boolean;
var
  i: Integer;
begin
  Result := False;
  for i := 1 to Count do
  begin
    if (Items[i - 1].Component = Component) then
    begin
      Result := true;
      Break;
    end;
  end;
end;

procedure TControlCollection.SetItem(Index: Integer;
  const Value: TControlItem);
begin
  inherited Items[Index] := Value;
end;

end.
