{***********************************************************************}
{ TMS Security System                                                   }
{ for Delphi & C++Builder                                               }
{                                                                       }
{ written by                                                            }
{   TMS Software                                                        }
{   Copyright  2003 - 2012                                             }
{   Email : info@tmssoftware.com                                        }
{   Web : http://www.tmssoftware.com                                    }
{***********************************************************************}
unit uIWSecDlg; { TuilSecurityDlg component }

{$I TMSDEFS.INC}

interface

uses
  Windows,
  SysUtils,
  Messages,
  Classes,
  Graphics,
  Controls,
  uSecControl,
  IWAppForm,
  uIWSecDlgForm,
  uIWEditUserForm,
  uIWEditGroupForm
  {$IFDEF TMSIW121}
  , IWCompTreeview
  {$ENDIF}
  {$IFNDEF TMSIW121}
  , IWTreeview
  {$ENDIF}
  ;

const
  REG_SettingsSectionName = 'Security Dialog';

type
  TuilIWSecurityDlg = class(TComponent)
  private
    { Private declarations }
    FSecurityManager: TuilSecurityManager;
    FFormCaption: string;
    FUsersCaption: string;
    FGroupsCaption: string;
    FMembershipCaption: string;
    FShowGroups: Boolean;

    FOnUserAdded: TuilUserEvent;
    FOnUserDeleted: TuilUserEvent;
    FOnUserEdited: TuilUserEvent;
    FOnGroupAdded: TuilGroupEvent;
    FOnGroupDeleted: TuilGroupEvent;
    FOnGroupEdited: TuilGroupEvent;

    FCanAddUser: Boolean;
    FCanDeleteUser: Boolean;
    FCanEditUser: Boolean;
    FCanAddGroup: boolean;
    FCanEditGroup: boolean;
    FCanDeleteGroup: boolean;
    procedure SetSecurityManager(newValue: TuilSecurityManager);
  protected
    { Protected declarations }
    procedure Notification(AComponent: TComponent; operation: TOperation); override;
    procedure Loaded; override;

    procedure TriggerUserAdded(Sender: TObject; UserName:string);
    procedure TriggerUserDeleted(Sender: TObject; UserName:string);
    procedure TriggerUserEdited(Sender: TObject; UserName:string);
    procedure TriggerGroupAdded(Sender: TObject; GroupName:string);
    procedure TriggerGroupDeleted(Sender: TObject; GroupName:string);
    procedure TriggerGroupEdited(Sender: TObject; GroupName:string);
    
    procedure RefreshUsers(AForm: TIWSecDlgForm);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
    function Execute: boolean;
  published
    { Published properties and events }
    property CanAddUser: Boolean read FCanAddUser Write FCanAddUser default True;
    property CanDeleteUser: Boolean read FCanDeleteUser Write FCanDeleteUser default True;
    property CanEditUser: Boolean read FCanEditUser Write FCanEditUser default True;
    property CanAddGroup: Boolean read FCanAddGroup Write FCanAddGroup default True;
    property CanDeleteGroup: Boolean read FCanDeleteGroup Write FCanDeleteGroup default True;
    property CanEditGroup: Boolean read FCanEditGroup Write FCanEditGroup default True;

    property FormCaption: string read FFormCaption write FFormCaption; { Published }
    property UsersCaption: string read FUsersCaption write FUsersCaption; { Published }
    property GroupsCaption: string read FGroupsCaption write FGroupsCaption; { Published }
    property MembershipCaption: string read FMembershipCaption write FMembershipCaption; { Published }
    property ShowGroups: Boolean read FShowGroups write FShowGroups default True; { Published }
    property SecurityManager: TuilSecurityManager read FSecurityManager write SetSecurityManager;

    property OnUserAdded: TuilUserEvent read FOnUserAdded write FOnUserAdded;
    property OnUserDeleted: TuilUserEvent read FOnUserDeleted write FOnUserDeleted;
    property OnUserEdited: TuilUserEvent read FOnUserEdited write FOnUserEdited;

    property OnGroupAdded: TuilGroupEvent read FOnGroupAdded write FOnGroupAdded;
    property OnGroupDeleted: TuilGroupEvent read FOnGroupDeleted write FOnGroupDeleted;
    property OnGroupEdited: TuilGroupEvent read FOnGroupEdited write FOnGroupEdited;
    
  end; { TuilIWSecurityDlg }

implementation

{$IFDEF FREEWARE}
uses
  IWKlooch;
{$ENDIF}


procedure TuilIWSecurityDlg.RefreshUsers(AForm: TIWSecDlgForm);
var
   Node,
   //SubNode : TTreeNode;
   SubNode: TIWTreeViewItem;
begin
   AForm.tvUsers.Items.Clear;
   AForm.tvUserMembers.Items.Clear;

   with SecurityManager.UserBindary.DataSource.DataSet do
   begin
      First;
      while not EOF do
      begin
         Node := AForm.tvUserMembers.Items.Add(nil);
         Node.Caption := FieldByName((SecurityManager.UserBindary.UserNameField)).AsString;
//         Node.ImageIndex := 0;
//        Node.SelectedIndex := 0;
         Node := AForm.tvUsers.Items.Add(nil);
         Node.Caption := FieldByName((SecurityManager.UserBindary.UserNameField)).AsString;
//         Node.ImageIndex := 0;
//         Node.SelectedIndex := 0;

         // Add Permissions...
         with SecurityManager.UserBindary.UserPermissions.DataSource.DataSet do
         begin
            First;
            while not EOF do
            begin
               SubNode := AForm.tvUsers.Items.Add(Node);
               SubNode.Caption := FieldByName((SecurityManager.UserBindary.UserPermissions.PermissionField)).AsString;
               //SubNode.ImageIndex := 2;
               //SubNode.SelectedIndex := 2;
               Next;
            end;
         end;
         Next;
      end;      // while
   end;      // with
//   tvUsers.FullExpand;
end;

function TuilIWSecurityDlg.Execute: boolean;
var
  AForm: TIWSecDlgForm;
begin
  Result := false;

  AForm := TIWSecDlgForm.Create(nil);
  SecurityManager.UserBindary.DataSource.DataSet.Open;
  SecurityManager.UserBindary.UserPermissions.DataSource.DataSet.Open;
  SecurityManager.PermissionBindary.DataSource.DataSet.Open;
  SecurityManager.GroupBindary.DataSource.DataSet.Open;
  SecurityManager.GroupBindary.GroupPermissions.DataSource.DataSet.Open;
  SecurityManager.GroupBindary.GroupMembershipBindary.DataSource.DataSet.Open;
  
  AForm.SecurityManager := FSecurityManager;

  with AForm do
  begin
    try
      OnUserAdded := TriggerUserAdded;
      OnUserDeleted := TriggerUserDeleted;
      OnUserEdited := TriggerUserEdited;

      OnGroupAdded := TriggerGroupAdded;
      OnGroupDeleted := TriggerGroupDeleted;
      OnGroupEdited := TriggerGroupEdited;

      IWTabControl1Page1.Visible := FShowGroups;
      IWTabControl1Page2.Visible := FShowGroups;

      IWTabControl1Page0.Title := FUsersCaption;
      IWTabControl1Page1.Title := FGroupsCaption;
      IWTabControl1Page2.Title := FMembershipCaption;
      Title := FFormCaption;
    finally
    end;  
  end;

  AForm.btAddUser.Enabled := FCanAddUser;
  AForm.btEditUser.Enabled := FCanEditUser;
  AForm.btRemove.Enabled := FCanDeleteUser;
  AForm.btAddGroup.Enabled := FCanAddGroup;
  AForm.btEditGroup.Enabled := FCanEditGroup;
  AForm.btRemoveGroup.Enabled := FCanDeleteGroup;
  AForm.Show;
end; 

procedure TuilIWSecurityDlg.SetSecurityManager(newValue: TuilSecurityManager);
begin
  if FSecurityManager <> newValue then
  begin
    FSecurityManager := newValue;
{$IFDEF UIL_WIN32}
    if newValue <> nil then
      newValue.FreeNotification(Self);
{$ENDIF}
  end;
end;


procedure TuilIWSecurityDlg.TriggerGroupAdded(Sender: TObject; GroupName: string);
begin
  if Assigned(OnGroupAdded) then
    OnGroupAdded(Self, GroupName);

end;

procedure TuilIWSecurityDlg.TriggerGroupDeleted(Sender: TObject;
  GroupName: string);
begin
  if Assigned(OnGroupDeleted) then
    OnGroupDeleted(Self, GroupName);
end;

procedure TuilIWSecurityDlg.TriggerGroupEdited(Sender: TObject;
  GroupName: string);
begin
  if Assigned(OnGroupEdited) then
    OnGroupEdited(Self, GroupName);
end;

procedure TuilIWSecurityDlg.TriggerUserAdded(Sender: TObject; UserName: string);
begin
  if Assigned(OnUserAdded) then
    OnUserAdded(Self, UserName);
end;

procedure TuilIWSecurityDlg.TriggerUserDeleted(Sender: TObject; UserName: string);
begin
  if Assigned(OnUserDeleted) then
    OnUserDeleted(Self, UserName);
end;

procedure TuilIWSecurityDlg.TriggerUserEdited(Sender: TObject; UserName: string);
begin
  if Assigned(OnUserEdited) then
    OnUserEdited(Self, UserName);
end;

{ SetSecurityManager }

procedure TuilIWSecurityDlg.Notification(AComponent: TComponent; operation: TOperation);
begin
  inherited Notification(AComponent, operation);
  if operation = opRemove then
  begin
    if AComponent = FSecurityManager then
      FSecurityManager := nil;
  end;
end; { Notification }

procedure TuilIWSecurityDlg.Loaded;
begin
  inherited Loaded;
end; { Loaded }

destructor TuilIWSecurityDlg.Destroy;
begin
  inherited Destroy;
end;

constructor TuilIWSecurityDlg.Create(AOwner: TComponent);
{ Creates an object of type TuilIWSecurityDlg, and initializes properties. }
begin
  inherited Create(AOwner);

  { Initialize properties with default values: }
  FShowGroups := True;
  FFormCaption := 'Security Settings';
  FUsersCaption := 'Users';
  FGroupsCaption := 'Groups';
  FMembershipCaption := 'Membership';

  FCanAddUser := true;
  FCanDeleteUser := true;
  FCanEditUser := true;
  FCanAddGroup := true;
  FCanDeleteGroup := true;
  FCanEditGroup := true;
end;

{$IFDEF FREEWARE}
initialization
   GLicense.ThirdPartyHook('TMS IntraWeb Components evaluation',True);
{$ENDIF}


end.
