{***********************************************************************}
{ TMS Security System                                                   }
{ for Delphi & C++Builder                                               }
{                                                                       }
{ written by                                                            }
{   TMS Software                                                        }
{   Copyright  2003 - 2012                                             }
{   Email : info@tmssoftware.com                                        }
{   Web : http://www.tmssoftware.com                                    }
{***********************************************************************}

unit uIWSecDlgForm;

{$I TMSDEFS.INC}

interface

uses
  Classes, SysUtils, IWAppForm, IWApplication, IWColor, IWTypes, IWCompButton,
  IWVCLBaseControl, IWBaseControl, IWBaseHTMLControl, IWControl,
  IWCompLabel, IWCompTabControl, Controls, Forms, IWVCLBaseContainer,
  IWContainer, IWHTMLContainer, IWHTML40Container, IWRegion,
  uSecControl, Dialogs, Windows, uIWEditUserForm, uIWEditGroupForm
  , DB, IWServer
  {$IFDEF DELPHI6_LVL}
  , Variants
  {$ENDIF}
  {$IFDEF TMSIW121}
  , IWCompTreeview, IWCompExtCtrls, IWServerInternalFiles
  {$ENDIF}
  {$IFNDEF TMSIW121}
  , IWTreeview, IWExtCtrls
  {$ENDIF}
  ;

type
  TUserState = (userEdit, userAdd);
  TGroupState = (groupEdit, groupAdd);

  TIWSecDlgForm = class(TIWAppForm)
    IWRegion2: TIWRegion;
    lbFormPolicy: TIWLabel;
    IWRegion1: TIWRegion;
    tabPermissions: TIWTabControl;
    IWTabControl1Page1: TIWTabPage;
    lbGroups: TIWLabel;
    tvGroups: TIWTreeView;
    btAddPermGroup: TIWButton;
    btRemovePermGroup: TIWButton;
    IWLabel2: TIWLabel;
    tvPermGroups: TIWTreeView;
    btAddGroup: TIWButton;
    btEditGroup: TIWButton;
    btRemoveGroup: TIWButton;
    btCloseGroup: TIWButton;
    IWTabControl1Page2: TIWTabPage;
    tvGroupMembers: TIWTreeView;
    IWLabel1: TIWLabel;
    btAddPermMembers: TIWButton;
    btRemovePermMembers: TIWButton;
    tvUserMembers: TIWTreeView;
    IWLabel3: TIWLabel;
    btRemoveMembers: TIWButton;
    btCloseMembers: TIWButton;
    IWTabControl1Page0: TIWTabPage;
    lbUsers: TIWLabel;
    lbPermissions: TIWLabel;
    tvUsers: TIWTreeView;
    tvPermissions: TIWTreeView;
    btAddUser: TIWButton;
    btEditUser: TIWButton;
    btRemove: TIWButton;
    btClose: TIWButton;
    btAddPermisson: TIWButton;
    btRemovePermission: TIWButton;
    IWImageFile1: TIWImageFile;
    procedure btCloseClick(Sender: TObject);
    procedure RefreshUsers;
    procedure RefreshGroups;
    procedure RefreshPermissions;
    procedure RefreshMembership;
    procedure IWAppFormRender(Sender: TObject);
    procedure btAddPermissonClick(Sender: TObject);
    procedure tvUsersTreeItemClick(Sender: TObject;
      ATreeViewItem: TIWTreeViewItem);
    procedure tvPermissionsTreeItemClick(Sender: TObject;
      ATreeViewItem: TIWTreeViewItem);
    procedure btRemovePermissionClick(Sender: TObject);
    procedure IWAppFormCreate(Sender: TObject);
    procedure tvPermGroupsTreeItemClick(Sender: TObject;
      ATreeViewItem: TIWTreeViewItem);
    procedure tvGroupsTreeItemClick(Sender: TObject;
      ATreeViewItem: TIWTreeViewItem);
    procedure btAddPermGroupClick(Sender: TObject);
    procedure btRemovePermGroupClick(Sender: TObject);
    procedure tvGroupMembersTreeItemClick(Sender: TObject;
      ATreeViewItem: TIWTreeViewItem);
    procedure tvUserMembersTreeItemClick(Sender: TObject;
      ATreeViewItem: TIWTreeViewItem);
    procedure btAddPermMembersClick(Sender: TObject);
    procedure btRemovePermMembersClick(Sender: TObject);
    procedure btAddUserClick(Sender: TObject);
    procedure btEditUserClick(Sender: TObject);
    procedure btEditGroupClick(Sender: TObject);
    procedure btAddGroupClick(Sender: TObject);
    procedure btRemoveMembersClick(Sender: TObject);
    procedure btRemoveClick(Sender: TObject);
  private
    FUserState: TUserState;
    FGroupState: TGroupState;
    FOnUserAdded: TuilUserEvent;
    FOnUserDeleted: TuilUserEvent;
    FOnUserEdited: TuilUserEvent;
    FOnGroupAdded: TuilGroupEvent;
    FOnGroupDeleted: TuilGroupEvent;
    FOnGroupEdited: TuilGroupEvent;
    
  public
    SecurityManager : TuilSecurityManager;
    IsPostBack: boolean;
    tvUsersIndex: integer;
    tvPermissionsIndex: integer;
    tvPermGroupsIndex: integer;
    tvGroupsIndex: integer;
    tvGroupMembersIndex: integer;
    tvUserMembersIndex: integer;
    procedure RefreshFromGroupSubForm(Sender: TObject; GroupName: string);
    procedure RefreshFromUserSubForm(Sender: TObject; UserName: string);
//    procedure Create(AOwner: TComponent; ASecurityManager: TuilSecurityManager); override;

    property OnUserAdded: TuilUserEvent read FOnUserAdded write FOnUserAdded;
    property OnUserDeleted: TuilUserEvent read FOnUserDeleted write FOnUserDeleted;
    property OnUserEdited: TuilUserEvent read FOnUserEdited write FOnUserEdited;

    property OnGroupAdded: TuilGroupEvent read FOnGroupAdded write FOnGroupAdded;
    property OnGroupDeleted: TuilGroupEvent read FOnGroupDeleted write FOnGroupDeleted;
    property OnGroupEdited: TuilGroupEvent read FOnGroupEdited write FOnGroupEdited;

  end;

implementation

{$R *.dfm}

{
procedure TIWSecDlgForm.Create(Sender: TComponent; ASecurityManager: TuilSecurityManager);
begin
  //inherited;
  //here
  SecurityManager := ASecurityManager;
end;
}

procedure TIWSecDlgForm.btAddGroupClick(Sender: TObject);
var
  AForm: TIWEditGroupForm;
begin
  AForm := TIWEditGroupForm.Create(nil);
  try
    AForm.edGroupName.DataField := SecurityManager.GroupBindary.GroupNameField;
    AForm.edGroupDescription.DataField := SecurityManager.GroupBindary.GroupDescriptionField;
    AForm.dsGroups.DataSet := SecurityManager.GroupBindary.DataSource.DataSet;
    AForm.dsGroups.DataSet.Append;
    AForm.OnChange := RefreshFromGroupSubForm;
  finally
  end;
  FGroupState := groupAdd;
  AForm.Show;
end;

procedure TIWSecDlgForm.btAddPermGroupClick(Sender: TObject);
var
   Node, SubNode : TIWTreeViewItem;
   myNames: TStringList;
begin
  if not Assigned(tvPermGroups.Selected) then
    Exit;

  Node := tvGroups.Selected;
  if not Assigned(Node) then
    Exit;

  while Assigned(Node.ParentItem) do
    Node := Node.ParentItem;

  if SecurityManager.GroupBindary.Locate(TStringList(Node.UserData)[0]) then
    if SecurityManager.GroupBindary.GroupPermissions.AddGroupAccess(
      TStringList(Node.UserData)[0], TStringList(tvPermGroups.Selected.UserData)[0]) then
    begin
      SubNode := tvGroups.Items.Add(Node);
      SubNode.Caption := TStringList(tvPermGroups.Selected.UserData)[0];
      myNames := TStringList.Create;
      myNames.Add(SubNode.Caption);
      SubNode.UserData := myNames;
      SubNode.ItemImages.DocumentImage.URL := '/files/permission.gif';
      SubNode.Name := tvGroups.HTMLName + IntToStr(SubNode.Index);
    end;
end;

procedure TIWSecDlgForm.btAddPermissonClick(Sender: TObject);
var
  Node, SubNode : TIWTreeViewItem;
  myNames: TStringList;
begin
  if not Assigned(tvPermissions.Selected) then
    Exit;

  Node := tvUsers.Selected;
  if not Assigned(Node) then
    Exit;

  while Assigned(Node.ParentItem) do
    Node := Node.ParentItem;

  if SecurityManager.UserBindary.Locate(TStringList(Node.UserData)[0]) then
    if SecurityManager.UserBindary.UserPermissions.AddUserAccess(
      TStringList(Node.UserData)[0], TStringList(tvPermissions.Selected.UserData)[0]) then
    begin
      //AddChild(tvUsers, Node, tvPermissions.Selected.Caption, 2);
      SubNode := tvUsers.Items.Add(Node);
      SubNode.Caption := TStringList(tvPermissions.Selected.UserData)[0];
      myNames := TStringList.Create;
      myNames.Add(SubNode.Caption);
      SubNode.UserData := myNames;
      SubNode.ItemImages.DocumentImage.URL := '/files/permission.gif';
      SubNode.Name := tvUsers.HTMLName + IntToStr(SubNode.Index);
    end;

   //Node.Selected := true;

   //Node := tvPermissions.Selected.GetNextSibling;

   //if Node <> nil then
   //   Node.Selected := true;
end;

procedure TIWSecDlgForm.btAddPermMembersClick(Sender: TObject);
var
   Node, SubNode : TIWTreeViewItem;
   myNames: TStringList;
begin
   if not Assigned(tvUserMembers.Selected) then
     Exit;

   Node := tvGroupMembers.Selected;

   if not Assigned(Node) then
     Exit;

   while Assigned(Node.ParentItem) do
      Node := Node.ParentItem;

   if SecurityManager.GroupBindary.Locate(TStringList(Node.UserData)[0]) then
   begin
      if SecurityManager.GroupBindary.GroupMembershipBindary.AddUserToGroup(
        TStringList(tvUserMembers.Selected.UserData)[0], TStringList(Node.UserData)[0]) then
      begin
         SubNode := tvGroupMembers.Items.Add(Node);
         SubNode.Caption := TStringList(tvUserMembers.Selected.UserData)[0];
         myNames := TStringList.Create;
         myNames.Add(SubNode.Caption);
         SubNode.UserData := myNames;
         SubNode.ItemImages.DocumentImage.URL := '/files/user.gif';
         SubNode.Name := tvGroupMembers.HTMLName + IntToStr(SubNode.Index);
      end;
   end;
end;

procedure TIWSecDlgForm.btAddUserClick(Sender: TObject);
var
  AForm: TIWEditUserForm;
  Fld: TField;
begin
  AForm := TIWEditUserForm.Create(nil);
  try
    AForm.edUsername.DataField := SecurityManager.UserBindary.UserNameField;
    AForm.edFullName.DataField := SecurityManager.UserBindary.FullNameField;
    AForm.edPassword.DataField := SecurityManager.UserBindary.PasswordField;
    AForm.lbAccessCount.DataField := SecurityManager.UserBindary.AccessCountField;
    AForm.lbCreatedDate.DataField := SecurityManager.UserBindary.CreatedDateField;
    AForm.lbLastAccess.DataField := SecurityManager.UserBindary.LastAccessField;
    AForm.cbEnabled.DataField := SecurityManager.UserBindary.EnabledField;
    AForm.dsUsers.DataSet := SecurityManager.UserBindary.DataSource.DataSet;
    AForm.SecurityManager := SecurityManager;
    AForm.OnChange := RefreshFromUserSubForm;

    with AForm.dsUsers.DataSet do
    begin
      Append;
      if SecurityManager.UserBindary.AccessCountField <> '' then
      begin
        if Assigned(FieldByName(SecurityManager.UserBindary.AccessCountField)) then
          FieldByName(SecurityManager.UserBindary.AccessCountField).AsInteger := 0;
      end;
      if SecurityManager.UserBindary.CreatedDateField <> '' then
      begin
        if Assigned(FieldByName(SecurityManager.UserBindary.CreatedDateField)) then
          FieldByName(SecurityManager.UserBindary.CreatedDateField).AsDateTime := Date;
      end;
      if SecurityManager.UserBindary.CreatedTimeField <> '' then
      begin
        if Assigned(FieldByName(SecurityManager.UserBindary.CreatedTimeField)) then
          FieldByName(SecurityManager.UserBindary.CreatedTimeField).AsDateTime := Time;
      end;
      if SecurityManager.UserBindary.EnabledField <> '' then
      begin
        if Assigned(FieldByName(SecurityManager.UserBindary.EnabledField)) then
        begin
          Fld := FieldByName(SecurityManager.UserBindary.EnabledField);
          case Fld.DataType of
          ftBoolean:
            Fld.AsBoolean := true;
          ftInteger:
            Fld.AsInteger := 1;
          ftString, ftWideString:
            Fld.AsString := 'T';
          end;
        end;
      end;
    end;  // with.Append;

  finally
  end;    
  
  FUserState := userAdd;
  AForm.Show;
end;

procedure TIWSecDlgForm.btCloseClick(Sender: TObject);
begin
  TIWAppForm(WebApplication.ActiveForm).Release;
end;
                                                

procedure TIWSecDlgForm.btEditGroupClick(Sender: TObject);
var
  AForm: TIWEditGroupForm;
begin
  if tvGroupsIndex >= 0 then
    tvGroups.Selected := tvGroups.Items[tvGroupsIndex];

  if tvGroups.Selected = nil then Exit;
  if tvGroups.Selected.ParentItem <> nil then Exit;

  if SecurityManager.GroupBindary.Locate(TStringList(tvGroups.Selected.UserData)[0]) then
  begin
    AForm := TIWEditGroupForm.Create(nil);
    try
      AForm.edGroupName.DataField := SecurityManager.GroupBindary.GroupNameField;
      AForm.edGroupDescription.DataField := SecurityManager.GroupBindary.GroupDescriptionField;
      AForm.dsGroups.DataSet := SecurityManager.GroupBindary.DataSource.DataSet;
      AForm.dsGroups.DataSet.Edit;
      AForm.OnChange := RefreshFromGroupSubForm;
      FGroupState := groupEdit;
      AForm.Show;
    finally
    end;
  end;
end;


procedure TIWSecDlgForm.btEditUserClick(Sender: TObject);
var
  AForm: TIWEditUserForm;
  OldPassword : string;
begin
  if tvUsersIndex >= 0 then
    tvUsers.Selected := tvUsers.Items[tvUsersIndex];

  if tvUsers.Selected = nil then
    Exit;

  if tvUsers.Selected.ParentItem <> nil then
    tvUsers.Selected := tvUsers.Selected.ParentItem;

  if SecurityManager.UserBindary.Locate(TStringList(tvUsers.Selected.UserData)[0]) then
  begin
    AForm := TIWEditUserForm.Create(nil);
    try
      OldPassword := SecurityManager.UserBindary.DataSource.DataSet.FieldByName(SecurityManager.UserBindary.PasswordField).AsString;
      AForm.edUsername.DataField := SecurityManager.UserBindary.UserNameField;
      AForm.edFullName.DataField := SecurityManager.UserBindary.FullNameField;
      AForm.edPassword.DataField := SecurityManager.UserBindary.PasswordField;
      AForm.lbAccessCount.DataField := SecurityManager.UserBindary.AccessCountField;
      AForm.lbCreatedDate.DataField := SecurityManager.UserBindary.CreatedDateField;
      AForm.lbLastAccess.DataField := SecurityManager.UserBindary.LastAccessField;
      AForm.cbEnabled.DataField := SecurityManager.UserBindary.EnabledField;
      AForm.dsUsers.DataSet := SecurityManager.UserBindary.DataSource.DataSet;
      AForm.edVerify.Text := OldPassword;
      AForm.OnChange := RefreshFromUserSubForm;
      FUserState := userEdit;

      with AForm.dsUsers.DataSet do
        Edit;

      AForm.Show;
    finally
    end;
  end;
end;

procedure TIWSecDlgForm.btRemoveClick(Sender: TObject);
var
  I: Integer;
  usr: string;
begin
  if tvUsers.Selected = nil then
    Exit;

  if tvUsers.Selected.ParentItem = nil then
  begin
    for I := 0 to tvUsers.Items.Count - 1 do
    begin
      SecurityManager.UserBindary.UserPermissions.RemoveUserAccess(
        TStringList(tvUsers.Selected.UserData)[0], TStringList(tvUsers.Items[i].UserData)[0]);
    end;

    usr := TStringList(tvUsers.Selected.UserData)[0];

    if SecurityManager.UserBindary.Locate(usr) then
    begin
      SecurityManager.UserBindary.DataSource.Dataset.Edit;
      SecurityManager.UserBindary.DataSource.Dataset.Delete;
      if Assigned(OnUserDeleted) then
        OnUserDeleted(Self,usr);
    end;

    tvUsersIndex := -1;
    RefreshUsers;
  end
   {
   else
   begin
      if SecurityManager.UserBindary.Locate(TStringList(tvUsers.Selected.ParentItem.UserData)[0]) then
      begin
         if SecurityManager.UserBindary.UserPermissions.RemoveUserAccess(
          TStringList(tvUsers.Selected.ParentItem.UserData)[0],TStringList(tvUsers.Selected.UserData)[0]) then
            tvUsers.Selected.Delete;
      end;
   end
   }
   ;
end;

procedure TIWSecDlgForm.btRemoveMembersClick(Sender: TObject);
begin
  if tvUserMembers.Selected = nil then exit;

  if SecurityManager.UserBindary.Locate(TStringList(tvUserMembers.Selected.UserData)[0]) then
  begin
    SecurityManager.UserBindary.DataSource.Dataset.Edit;
    SecurityManager.UserBindary.DataSource.Dataset.Delete;
    RefreshUsers;
  end;
end;

procedure TIWSecDlgForm.btRemovePermGroupClick(Sender: TObject);
var
  i: integer;
  grp: string;
begin
  if tvGroups.Selected = nil then
    Exit;

  if tvGroups.Selected.ParentItem = nil then
  begin
    for I := 0 to tvGroups.Items.Count - 1 do
    begin
      SecurityManager.GroupBindary.GroupPermissions.RemoveGroupAccess(
        TStringList(tvGroups.Selected.UserData)[0], TStringList(tvGroups.Items[i].UserData)[0]);
    end;

    grp := TStringList(tvGroups.Selected.UserData)[0];
    if SecurityManager.GroupBindary.Locate(grp) then
    begin
      SecurityManager.GroupBindary.DataSource.Dataset.Edit;
      SecurityManager.GroupBindary.DataSource.Dataset.Delete;

      if Assigned(OnGroupDeleted) then
        OnGroupDeleted(Self,grp);
    end;
    tvGroupsIndex := -1;
    RefreshGroups;
  end
  else
  begin
    if SecurityManager.GroupBindary.Locate(TStringList(
       tvGroups.Selected.ParentItem.UserData)[0]) then
    begin
      if SecurityManager.GroupBindary.GroupPermissions.RemoveGroupAccess(
        TStringList(tvGroups.Selected.ParentItem.UserData)[0],TStringList(tvGroups.Selected.UserData)[0]) then
      begin
        tvGroups.Selected.Destroy;
        tvGroups.Selected := nil;
        tvGroupsIndex := -1;
      end;
    end;
  end;
end;

procedure TIWSecDlgForm.btRemovePermissionClick(Sender: TObject);
begin
   if tvUsers.Selected = nil then exit;

   if tvUsers.Selected.ParentItem = nil then
   begin
         if SecurityManager.UserBindary.Locate(TStringList(tvUsers.Selected.UserData)[0]) then
         begin
            SecurityManager.UserBindary.DataSource.Dataset.Edit;
            SecurityManager.UserBindary.DataSource.Dataset.Delete;

//            if Assigned(OnUserDeleted) then
//              OnUserDeleted(Self,tvUsersUsers.Selected.Text);

         end;
         RefreshUsers;
      //end;
   end
   else
   begin
      if SecurityManager.UserBindary.Locate(TStringList(tvUsers.Selected.ParentItem.UserData)[0]) then
      begin
         if SecurityManager.UserBindary.UserPermissions.RemoveUserAccess(
          TStringList(tvUsers.Selected.ParentItem.UserData)[0],TStringList(tvUsers.Selected.UserData)[0]) then
         begin
            tvUsers.Selected.Destroy;
            tvUsers.Selected := nil;
            tvUsersIndex := -1;
         end;
      end;
   end;
end;

procedure TIWSecDlgForm.btRemovePermMembersClick(Sender: TObject);
begin
   if (tvGroupMembers.Selected = nil) then Exit;
   if (tvGroupMembers.Selected.ParentItem = nil) then Exit;

      SecurityManager.GroupBindary.GroupMembershipBindary.RemoveUserFromGroup(
        TStringList(tvGroupMembers.Selected.UserData)[0], TStringList(tvGroupMembers.Selected.ParentItem.UserData)[0]);

   RefreshGroups;
   RefreshMembership;
end;

procedure TIWSecDlgForm.IWAppFormCreate(Sender: TObject);
begin
  tabPermissions.ActivePage := 0;
  IsPostBack := false;
  tvUsersIndex := -1;
  tvPermissionsIndex := -1;
  tvPermGroupsIndex := -1;
  tvGroupsIndex := -1;
  tvGroupMembersIndex := -1;
  tvUserMembersIndex := -1;

  {$IFDEF TMSIW121}
  gInternalFiles.add('IW_GFX_user', '/files/disable.gif');
  gInternalFiles.add('IW_GFX_groups', '/files/hide.gif');
  gInternalFiles.add('IW_GFX_permission', '/files/permission.gif');
  gInternalFiles.add('IW_GFX_userpermission', '/files/userpermission.gif');
  {$ELSE}
  TIWServer.AddInternalFile('IW_GFX_user', '/files/disable.gif');
  TIWServer.AddInternalFile('IW_GFX_groups', '/files/hide.gif');
  TIWServer.AddInternalFile('IW_GFX_permission', '/files/permission.gif');
  TIWServer.AddInternalFile('IW_GFX_userpermission', '/files/userpermission.gif');
  {$ENDIF}
end;

procedure TIWSecDlgForm.IWAppFormRender(Sender: TObject);
begin
  if not IsPostBack then
  begin
    RefreshUsers;
    RefreshGroups;
    RefreshPermissions;
    RefreshMembership;
    IsPostBack := true;
  end;

  if tvUsersIndex < 0 then
  begin
    btRemove.Enabled := false;
    btRemovePermission.Enabled := false;
    btEditUser.Enabled := false;
  end
  else
  begin
    if tvUsers.Items[tvUsersIndex].ParentItem = nil then
    begin
      btRemove.Enabled := true;
      btRemovePermission.Enabled := false
    end
    else
    begin
      btRemove.Enabled := false;
      btRemovePermission.Enabled := true;
    end;

    btEditUser.Enabled := true;
  end;

  if (tvPermissionsIndex < 0) or (tvUsersIndex < 0) then
    btAddPermisson.Enabled := false
  else
    btAddPermisson.Enabled := true;
    

  if tvGroupsIndex < 0 then
  begin
    btRemoveGroup.Enabled := false;
    btRemovePermGroup.Enabled := false;
    btEditGroup.Enabled := false;
  end
  else
  begin
    if tvGroups.Items[tvGroupsIndex].ParentItem = nil then
    begin
      btRemoveGroup.Enabled := true;
      btRemovePermGroup.Enabled := false;
    end
    else
    begin
      btRemoveGroup.Enabled := false;
      btRemovePermGroup.Enabled := true;
    end;

    btEditGroup.Enabled := true;
  end;

  if (tvGroupsIndex < 0) or (tvPermGroupsIndex < 0) then
    btAddPermGroup.Enabled := false
  else
    btAddPermGroup.Enabled := true;

  if tvUserMembersIndex < 0 then
  begin
     btRemoveMembers.Enabled := false;
  end
  else
  begin
     btRemoveMembers.Enabled := true;
  end;

  if tvGroupMembersIndex < 0 then
  begin
     btRemovePermMembers.Enabled := false;
  end
  else
  begin
     btRemovePermMembers.Enabled := true;
  end;

  if (tvGroupMembersIndex < 0) or (tvUserMembersIndex < 0) then
    btAddPermMembers.Enabled := false
  else
    btAddPermMembers.Enabled := true;

end;

procedure TIWSecDlgForm.RefreshFromGroupSubForm(Sender: TObject; GroupName: string);
begin
  RefreshUsers;
  RefreshGroups;
  case FGroupState of
  groupAdd: if Assigned(OnGroupAdded) then
              OnGroupAdded(self, GroupName);
  groupEdit: if Assigned(OnGroupEdited) then
               OnGroupEdited(Self, GroupName);
  end;
end;

procedure TIWSecDlgForm.RefreshFromUserSubForm(Sender: TObject; UserName: string);
begin
  RefreshUsers;
  RefreshGroups;
  case FUserState of
  userAdd: if Assigned(OnUserAdded) then
              OnUserAdded(self, UserName);
  userEdit: if Assigned(OnUserEdited) then
               OnUserEdited(Self, UserName);
  end;
end;


procedure TIWSecDlgForm.RefreshGroups;
var
   SubNode,
   Node: TIWTreeViewItem;
   myNames: TStringList;
begin
   tvGroups.Items.Clear;
   tvGroupMembers.Items.Clear;

   with SecurityManager.GroupBindary.DataSource.DataSet do
   begin
      First;
      while not EOF do
      begin
         Node := tvGroupMembers.Items.Add(nil);
         Node.Caption := FieldByName((SecurityManager.GroupBindary.GroupNameField)).AsString;
         myNames := TStringList.Create;
         myNames.Add(Node.Caption);
         Node.UserData := myNames;
         Node.Name := tvGroupMembers.HTMLName + IntToStr(Node.Index);

         Node.ItemImages.OpenFolderImage.URL := '/files/groups.gif';
         Node.ItemImages.ClosedFolderImage.URL := Node.ItemImages.OpenFolderImage.URL;
         Node.ItemImages.DocumentImage.URL := Node.ItemImages.OpenFolderImage.URL;

         Node := tvGroups.Items.Add(nil);
         Node.Caption := FieldByName((SecurityManager.GroupBindary.GroupNameField)).AsString;
         myNames := TStringList.Create;
         myNames.Add(Node.Caption);
         Node.UserData := myNames;
         Node.Name := tvGroups.HTMLName + IntToStr(Node.Index);

         Node.ItemImages.OpenFolderImage.URL := '/files/groups.gif';
         Node.ItemImages.ClosedFolderImage.URL := Node.ItemImages.OpenFolderImage.URL;
         Node.ItemImages.DocumentImage.URL := Node.ItemImages.OpenFolderImage.URL;

         // Add Permissions...
         with SecurityManager.GroupBindary.GroupPermissions.DataSource.DataSet do
         begin
            First;
            while not EOF do
            begin

               if FieldByName((SecurityManager.GroupBindary.GroupPermissions.GroupNameField)).AsString = TStringList(Node.UserData)[0] then
               begin
                 SubNode := tvGroups.Items.Add(Node);
                 SubNode.Caption := FieldByName((SecurityManager.GroupBindary.GroupPermissions.PermissionField)).AsString;
                 myNames := TStringList.Create;
                 myNames.Add(SubNode.Caption);
                 SubNode.UserData := myNames;
                 SubNode.Name := tvGroups.HTMLName + IntToStr(SubNode.Index);

                 SubNode.ItemImages.DocumentImage.URL := '/files/permission.gif';

               end;
               Next;
            end;
         end;
         Next;
      end;      // while
   end;      // with
end;

procedure TIWSecDlgForm.RefreshMembership;
var
   Node, SubNode,
   UserNode : TIWTreeViewItem;
   i, j: integer;
   myNames: TStringList;
begin

   for I := 0 to tvGroupMembers.Items.Count - 1 do
   begin
      Node := tvGroupMembers.Items[i];
      for j := 0 to tvUserMembers.Items.Count - 1 do
      begin
         UserNode := tvUserMembers.Items[j];
         if SecurityManager.GroupBindary.GroupMembershipBindary.Locate(
            VarArrayOf([TStringList(UserNode.UserData)[0], TStringList(Node.UserData)[0]])) then
         begin
            SubNode := tvGroupMembers.Items.Add(Node);
            SubNode.Caption := TStringList(UserNode.UserData)[0];
            myNames := TStringList.Create;
            myNames.Add(SubNode.Caption);
            SubNode.UserData := myNames;
            SubNode.Name := tvGroupMembers.HTMLName + IntToStr(SubNode.Index);

            SubNode.ItemImages.DocumentImage.URL := '/files/user.gif';

         end;
      end;      // for
   end;      // for
end;

procedure TIWSecDlgForm.RefreshPermissions;
var
   Node : TIWTreeViewItem;
   myNames: TStringList;
begin
   tvPermissions.Items.Clear;
   tvPermGroups.Items.Clear;

   with SecurityManager.PermissionBindary.DataSource.DataSet do
   begin
      First;
      while not EOF do
      begin
         Node := tvPermissions.Items.Add(nil);
         Node.Caption := FieldByName((SecurityManager.PermissionBindary.KeyField)).AsString;
         myNames := TStringList.Create;
         myNames.Add(Node.Caption);
         Node.UserData := myNames;
         Node.Name := tvPermissions.HTMLName + IntToStr(Node.Index);

         Node.ItemImages.DocumentImage.URL := '/files/permission.gif';

         Node := tvPermGroups.Items.Add(nil);
         Node.Caption := FieldByName((SecurityManager.PermissionBindary.KeyField)).AsString;
         myNames := TStringList.Create;
         myNames.Add(Node.Caption);
         Node.UserData := myNames;
         Node.Name := tvPermGroups.HTMLName + IntToStr(Node.Index);

         Node.ItemImages.DocumentImage.URL := '/files/permission.gif';
         Next;
      end;      // while
   end;      // with
end;

procedure TIWSecDlgForm.RefreshUsers;
var
   Node,
   SubNode: TIWTreeViewItem;
   myNames: TStringList;
begin
   tvUsers.Items.Clear;
   tvUserMembers.Items.Clear;

   with SecurityManager.UserBindary.DataSource.DataSet do
   begin
      First;
      while not EOF do
      begin
         Node := tvUserMembers.Items.Add(nil);
         Node.Caption := FieldByName((SecurityManager.UserBindary.UserNameField)).AsString;
         myNames := TStringList.Create;
         myNames.Add(Node.Caption);
         Node.UserData := myNames;
         Node.Name := tvUserMembers.HTMLName + IntToStr(Node.Index);

         Node.ItemImages.DocumentImage.URL := '/files/user.gif';

         Node := tvUsers.Items.Add(nil);
         Node.Caption := FieldByName((SecurityManager.UserBindary.UserNameField)).AsString;
         myNames := TStringList.Create;
         myNames.Add(Node.Caption);
         Node.UserData := myNames;
         Node.Name := tvUsers.HTMLName + IntToStr(Node.Index);

         Node.ItemImages.OpenFolderImage.URL := '/files/user.gif';
         Node.ItemImages.ClosedFolderImage.URL := Node.ItemImages.OpenFolderImage.URL;
         Node.ItemImages.DocumentImage.URL := Node.ItemImages.OpenFolderImage.URL;         

         // Add Permissions...
         with SecurityManager.UserBindary.UserPermissions.DataSource.DataSet do
         begin
            First;
            while not EOF do
            begin
               if FieldByName((SecurityManager.UserBindary.UserPermissions.UserNameField)).AsString = TStringList(Node.UserData)[0] then
               begin
                 SubNode := tvUsers.Items.Add(Node);
                 SubNode.Caption := FieldByName((SecurityManager.UserBindary.UserPermissions.PermissionField)).AsString;
                 myNames := TStringList.Create;
                 myNames.Add(SubNode.Caption);
                 SubNode.UserData := myNames;
                 SubNode.Name := tvUsers.HTMLName + IntToStr(SubNode.Index);

                 SubNode.ItemImages.DocumentImage.URL := '/files/permission.gif';

               end;
               Next;
            end;
         end;
         Next;
      end;      // while
   end;      // with
end;

procedure TIWSecDlgForm.tvGroupMembersTreeItemClick(Sender: TObject;
  ATreeViewItem: TIWTreeViewItem);
begin
  if tvGroupMembersIndex >= 0 then
  begin
    with tvGroupMembers.Items[tvGroupMembersIndex] do
    begin
      Font.Color := clWebBLUE;
      Caption := TStringList(UserData)[0];
    end;
  end;

  tvGroupMembersIndex := ATreeViewItem.Index;
  tvGroupMembers.Selected := ATreeViewItem;
  tvGroupMembers.ScrollToName := ATreeViewItem.Name;

  with tvGroupMembers.Selected do
  begin
    Font.Color := clWebWhite;
    Caption := '<span style="background-color:blue">' + TStringList(UserData)[0] + '</span>';
  end;
end;

procedure TIWSecDlgForm.tvGroupsTreeItemClick(Sender: TObject;
  ATreeViewItem: TIWTreeViewItem);
begin
  if tvGroupsIndex >= 0 then
  begin
    with tvGroups.Items[tvGroupsIndex] do
    begin
      Font.Color := clWebBLUE;
      Caption := TStringList(UserData)[0];
    end;
  end;

  tvGroupsIndex := ATreeViewItem.Index;
  tvGroups.Selected := ATreeViewItem;
  tvGroups.ScrollToName := ATreeViewItem.Name;

  with tvGroups.Selected do
  begin
    Font.Color := clWebWhite;
    Caption := '<span style="background-color:blue">' + TStringList(UserData)[0] + '</span>';
  end;
end;

procedure TIWSecDlgForm.tvPermGroupsTreeItemClick(Sender: TObject;
  ATreeViewItem: TIWTreeViewItem);
begin
  if tvPermGroupsIndex >= 0 then
  begin
    with tvPermGroups.Items[tvPermGroupsIndex] do
    begin
      Font.Color := clWebBLUE;
      Caption := TStringList(UserData)[0];
    end;
  end;

  tvPermGroupsIndex := ATreeViewItem.Index;
  tvPermGroups.Selected := ATreeViewItem;
  tvPermGroups.ScrollToName := ATreeViewItem.Name;

  with tvPermGroups.Selected do
  begin
    Font.Color := clWebWhite;
    Caption := '<span style="background-color:blue">' + TStringList(UserData)[0] + '</span>';
  end;
end;

procedure TIWSecDlgForm.tvPermissionsTreeItemClick(Sender: TObject;
  ATreeViewItem: TIWTreeViewItem);
begin
  if tvPermissionsIndex >= 0 then
  begin
    with tvPermissions.Items[tvPermissionsIndex] do
    begin
      Font.Color := clWebBLUE;
      Caption := TStringList(UserData)[0];
    end;
  end;

  tvPermissionsIndex := ATreeViewItem.Index;
  tvPermissions.Selected := ATreeViewItem;
  tvPermissions.ScrollToName := ATreeViewItem.Name;

  with tvPermissions.Selected do
  begin
    Font.Color := clWebWhite;
    Caption := '<span style="background-color:blue">' + TStringList(UserData)[0] + '</span>';
  end;
end;

procedure TIWSecDlgForm.tvUserMembersTreeItemClick(Sender: TObject;
  ATreeViewItem: TIWTreeViewItem);
begin
  if tvUserMembersIndex >= 0 then
  begin
    with tvUserMembers.Items[tvUserMembersIndex] do
    begin
      Font.Color := clWebBLUE;
      Caption := TStringList(UserData)[0];
    end
  end;

  tvUserMembersIndex := ATreeViewItem.Index;
  tvUserMembers.Selected := ATreeViewItem;
  tvUserMembers.ScrollToName := ATreeViewItem.Name;

  with tvUserMembers.Selected do
  begin
    Font.Color := clWebWhite;
    Caption := '<span style="background-color:blue">' + TStringList(UserData)[0] + '</span>';
  end;
end;

procedure TIWSecDlgForm.tvUsersTreeItemClick(Sender: TObject;
  ATreeViewItem: TIWTreeViewItem);
begin
  if tvUsersIndex >= 0 then
  begin
    with tvUsers.Items[tvUsersIndex] do
    begin
      Font.Color := clWebBlue;
      Caption := TStringList(UserData)[0];
    end;
  end;

  tvUsersIndex := ATreeViewItem.Index;
  tvUsers.Selected := ATreeViewItem;
  tvUsers.ScrollToName := ATreeViewItem.Name;

  with ATreeViewItem do
  begin
    Font.Color := clWebWhite;
    Caption := '<span style="background-color:blue">' + TStringList(UserData)[0] + '</span>';
  end;

end;

end.
